function [bonded, area, circumference] = contactAnalysisCS(I)
% Analyse a single cross section to determine if the fibre 
% is bonded or free. Algoritm I in the paper

generateEvaluationPictures = 0;

% Classification parameters
maxOuterFibreRadius = 50;
maxFibreWallThickness = 30;
maxArea = pi^2*(maxOuterFibreRadius-(maxOuterFibreRadius-maxFibreWallThickness));
maxCircumference = 2*pi*maxOuterFibreRadius;

% Preparations
r = (size(I,1)-1)/2;
[u,v]=meshgrid(-r:r,-r:r);
R = (u.^2+v.^2).^(1/2);

    threshold = 128;
    % Threshold and label components
    L=I; L(I<threshold)=0; L(L>0)=1;
    [LAB, nlab] = bwlabel(L); % Labelled    
    
    % The object has the label that can be found closest to the origin
    maxval = 0; maxlab = -1;
    for ll = 1:nlab
        lv = max(max((100-R).*(LAB==ll)));
        if lv > maxval
            maxval=lv; maxlab=ll;
        end
    end
    
    Obj = (LAB==maxlab); % Binary image
        
    % See if the object is in contact with the edge
    edgeContact=sum(sum(Obj(:,1))) + sum(sum(Obj(:,end))) + ...
        sum(sum(Obj(1,:))) + sum(sum(Obj(end,:)));
    
    % Area       
    area=pcArea(I, Obj(:), 255); 
    circumference=PerimeterEst(Obj, 1);
    
    bonded = 0; % i.e. free if not...
    if edgeContact || (area>maxArea) || (circumference>maxCircumference)
        bonded = 1;
    end        
    
    if area<4
        bonded = -1;
    end
    
    if generateEvaluationPictures        
        if bonded    
            nfiles = numel(dir('evaluation/alg1/contact/*png'));        
            imwrite(I/255, sprintf('evaluation/alg1/contact/%03d.png', nfiles+1));
        else
            nfiles = numel(dir('evaluation/alg1/free/*png'));        
            imwrite(I/255, sprintf('evaluation/alg1/free/%03d.png', nfiles+1));
        end
    end                 
end