%% Use the frenet frame to show the fibre cross sections along the fibre

% Fibre: the label closest to the middle
% Contact: If the label stretches to the boundary, else: not

threshold = 105; % Above is fibre, below, air

dans = inputdlg(['Enter fibre nr (1 -- ' num2str(numel(fibres)) ') or nothing for the current unsaved'], 'transversalMovie');
dans = dans{1};
if strcmp(dans, '')
    fibre = 0;
else
    fibre= str2num(dans);
end

if fibre == 0
    txx = xx;
    tyy = yy;
    tzz = zz;
else
    txx = fibres{fibre}.xx;
    tyy = fibres{fibre}.yy;
    tzz = fibres{fibre}.zz;
end

[T,N,B]=frenet(txx,tyy,tzz);

if 0
figure,
plot3(xx,yy,zz, 'b')
hold on
for kk=1:numel(xx)
    plot3([txx(kk), txx(kk)+n(kk,1)], [tyy(kk), tyy(kk)+n(kk,2)],[tzz(kk), tzz(kk)+n(kk,3)], 'k');
    plot3([txx(kk), txx(kk)+b(kk,1)], [tyy(kk), tyy(kk)+b(kk,2)],[tzz(kk), tzz(kk)+b(kk,3)], 'r');
end
axis equal
end


r = 20;

recordFibre = 0;
if recordFibre
    filename = sprintf('fibre%d_contact.avi', fibre);
    writerObj = VideoWriter(filename)
    writerObj.FrameRate = 20;
    open(writerObj);
end

figure(7)   
clf
axis image
%colormap gray

imagesc(ones(2*r+1,2*r+1));
im = get(gca, 'Children');
colormap gray

hold on
plot(r+1,r+1, 'rx', 'MarkerSize', 20)

set(gca, 'clim', threshold+[-5, 5])
textbox = text(20,5,'..', 'Color', 'k', 'FontSize', 20, 'BackgroundColor',[.7 .9 .7]);
textbox = text(20,5, '..', 'Color', 'k', 'FontSize', 20, 'BackgroundColor',[.7 .9 .7], 'Margin', 5);

[X,Y]=meshgrid(-r:r,-r:r);
R = (X.^2+Y.^2).^(1/2);

for kk=1:numel(txx)
    
    t = T(kk,:);
    n = N(kk,:);
    b = B(kk,:);
    
    [u,v]=meshgrid(-r:r,-r:r);
    
    PX = txx(kk)+u*n(1)+v*b(1); 
    PY = tyy(kk)+u*n(2)+v*b(2);
    PZ = tzz(kk)+u*n(3)+v*b(3);
    
    Skk= trilinear8(V, PY, PZ, PX );
    Skk=double(Skk);
    L=Skk;
    L(Skk<threshold)=0; L(L>0)=1;
        
    [LAB, nlab] = bwlabel(L); % Labelled    
    maxval = 0; maxlab = -1;
    for ll = 1:nlab
        lv = max(max((100-R).*(LAB==ll)));
        if lv > maxval
            maxval=lv; maxlab=ll;
        end
    end
    
    Obj = (LAB==maxlab);
    %figure, imagesc(Obj);
    
    Contact(kk)=sum(sum(Obj(:,1))) + sum(sum(Obj(:,end))) + ...
        sum(sum(Obj(1,:))) + sum(sum(Obj(end,:)));
    
    set(im, 'CData', Skk)        
    
    if(Contact(kk)==0)
        set(textbox, 'String', 'Free ');
    else
         set(textbox, 'String', 'Bonded');
    end
    %drawnow
    if recordFibre
        writeVideo(writerObj,getframe);    
    else
        pause(0.05)
    end
            
end

if recordFibre
    close(writerObj);
end

figure(11), 
plot(Contact>0)
title('Contact or free')

figure,
plot(area)
xlabel('position')
ylabel('area')

