/* see appleton2006globally
 * g is set constant = 1, to use variable metric, change 
 * g to g[pos] etc...
 *
 * The outer bondary of label has to be 0
 */


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <assert.h>
//#include <omp.h>
//#include <mex.h>

#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))
#define g 1

int flow3(double * label, double *pressure, double *fx, double *fy, double *fz, int m, int n,int p, int maxiter)
{

  // label: 0 = void, 1 = material, 2=object source, 3 = object sink
  // p: initial pressure
  // fx, fy : initial flow, set to 0
  // m,n size of the arrays
  // maxiter, maximum number of iterations

  double delta = 0.1; // time step
  double gamma = 0.15; // pixel convergence tollerence
  double mu = 0.99; // percentage of converged pixels
  int iterc = 0; // Check for convergence every 100th iteration
  int convergence = 0;
  double nset=0; double npositions = 0;
  int iter = 0;

  const int mn  = m*n;

//omp_set_num_threads(4);

//int nthreads = omp_get_num_threads();
//int maxthreads = omp_get_max_threads();
  //    printf(" %d  / %d threads\n",nthreads, maxthreads);


  // Main loop, until convergence
  do { 

    // fx and fy is always 0 where label == 0
    // p is undefined where label == 0 

    // Eq. 6
    #pragma OMP parallel for private(pos)
    for(int kk=1; kk<(m-1); kk++)
      for (int ll=1; ll<(n-1); ll++)	
        for (int jj=1; jj<(p-1); jj++) {
        int pos=kk+ll*m+jj*mn;
        if(label[pos]>0)
          pressure[pos]=pressure[pos]-delta*((fx[pos] -fx[pos-1]) + 
                               (fy[pos] -fy[pos-m]) + 
                               (fz[pos]-fz[pos-mn]));
      }

    // Eq. 7    
    for(int kk=0; kk<(m-1); kk++)
      for (int ll=0; ll<(n-1); ll++)	
        for (int jj=0; jj<(p-1); jj++)
      {
        int pos=kk+ll*m+jj*mn;
        if(label[pos]>0) // If the node which the edge is connected to is in use
        {
          if(label[pos+1]>0) // If there is a node to the right
          {
            fx[pos]=fx[pos]-delta*(pressure[pos+1]-pressure[pos]);
          } 
          if(label[pos+m]>0) // If there is a node below
          {
            fy[pos]=fy[pos]-delta*(pressure[pos+m]-pressure[pos]);
          }
          if(label[pos+mn]>0)
          {
            fz[pos]=fz[pos]-delta*(pressure[pos+mn]-pressure[pos]);            
          }
        }
      }
    
    // Eq. 8 and details on page 111
    for(int kk=1; kk<(m-1); kk++)
      for (int ll=1; ll<(n-1); ll++)	
        for (int jj=1; jj<(p-1); jj++)
      {
        int pos=kk+ll*m+jj*mn;
        if(label[pos]>0)
        {
          // Step 1
          double fxm;
          if (label[pos-1]>0)
          { fxm = MAX(MAX(-fx[pos-1], 0),fx[pos]); }
          else {  fxm = MAX(0,fx[pos]); }

          double fym; 
          if (label[pos-m]>0)
          { fym = MAX(MAX(-fy[pos-m], 0),fy[pos]); }
          else 
          { fym = MAX(0, fy[pos]); }

          double fzm; 
          if (label[pos-mn]>0)
          { fzm = MAX(MAX(-fz[pos-mn], 0),fz[pos]); }
          else 
          { fzm = MAX(0, fz[pos]); }
        
          // Step 2 -- induces oscillations?
          double v = sqrt(fxm*fxm +fym*fym+fzm*fzm); // Max outward velocity
          if(v>g) 
          {
            fxm = fxm*g/v; fym = fym*g/v; fzm = fzm*g/v;
            // Step 3  
            // -- requires another layer to store the update values... ?
            if (label[pos-1]>0)
              fx[pos-1]=MAX(fx[pos-1], -fxm);
            fx[pos] = MIN(fx[pos],   fxm);
            if (label[pos-m]>0)
              fy[pos-m]=MAX(fy[pos-m], -fym);
            fy[pos]  =MIN(fy[pos],    fym);
            if (label[pos-mn]>0)
              fz[pos-mn]=MAX(fz[pos-mn], -fzm);
            fz[pos]  =MIN(fz[pos],    fzm);             
          }
        }
      }

    // Set the pressure correct at the labels
    for(int kk=0; kk<m*n*p; kk++)
    {
      if(label[kk]==2)	
        pressure[kk]=1;
      if(label[kk]==3)
        pressure[kk]=0;
}

if(0) {  
  // Constrain min and max pressure
  // width delta = 0.1:
  // For the x.png 11615 with, 12827 without
  // for the barrier: 1111 with 20705 without 
  for(int kk=0; kk<m*n*p; kk++)
  {
    if(pressure[kk]>1)
      pressure[kk]=1;
    if(pressure[kk]<0)
      pressure[kk]=0;
  }
}   

    // Check for convergence every 100th iteration
    if(iterc==100)
    {
    //mexEvalString("refresh; pause(0.005)");
      iterc = 0;
      npositions=0; nset=0;
      for(int kk=0; kk<m*n*p; kk++)
      {
        if(label[kk]>0)
        {
          npositions++;
          if(fabs(pressure[kk]-1)<gamma | fabs(pressure[kk])<gamma)
            nset++;
        }
      }
      if ((nset/npositions)>mu)
        convergence = 1;
    } else
    { iterc++; }

    
    // See if too many iterations are used
    if (iter==maxiter)
      convergence = 1;

  iter++;
  } while (convergence == 0);
  // end main loop
  
  printf("%d, %f\n", iter, nset/npositions);
  return iter;
}




