/* 

Routines to convert between the different indexing schemes used by
matlab for matrixes and and the 1D vectors in C++ that arises when data 
is accessed by the mex-interface.

A few of the routines returns -1 if there is no element available.

Usage:
the variables xres, yres and zres has to be defined globally before 
importing this file.

Latest updated: 25 November 2008.


*/

int m2c(int, int, int); // convert matlab style to c-style returns -1 if out of bounds.

int c2mx(int); // c style to matlab x coordinate
int c2my(int); // c style to matlab y coordinate
int c2mz(int); // c style to matlab z coordinate

int xp(int); // x+1  Position functions. Returns -1 if no element there
int xm(int); // x-1
int yp(int); // y+1
int ym(int); // y-1
int zp(int); // z+1
int zm(int); // z-1


int m2c(int y, int x, int z)
{
  // converts matlab matrix indices to cpp vector index number
  if (x>0 && x<xres+1 && y>0 && y<yres+1 && z>0 && z < zres+1)
    {
      return y-1 + (x-1)*yres + (z-1)*xres*yres;
    }
  else
    {
      return -1;
    }
}

int c2mx(int c)
{
  return ((c)%(xres*yres)) / (yres)+1;
}

int c2my(int c)
{
  return c%yres+1;
}

int c2mz(int c){
  return (int) floor((c)/(yres*xres))+1;
}

int yp(int p)
{ // ok
  if (p==-1 or c2my(p)==yres)
    { return -1; }
  else {return p+1; }
}
int ym(int p)
{ // ok
  if (p==-1 or c2my(p)==1)
    {return -1;}
  else { return p-1;}
}

int xp(int p)
{ // ok
  if (p==-1 or c2mx(p)==xres)
    { return -1;} 
  else
    { return p+yres;} 
}
int xm(int p)
{ // ok
  if (p==-1 or c2mx(p)==1)
    { return -1; }
  else { return p-yres;}
}
int zp(int p)
{ // ok
  if(p==-1 or c2mz(p)==zres)
    {return -1; }
  else {return p+xres*yres;}
}
int zm(int p)
{ // ok
  if(p==1 or c2mz(p)==1)
    {return -1;}
  else{ return p-xres*yres;}
}

