// Example of a trilinear interpolation as a MEX file
#include "mex.h"
#include "flow3.h"

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
    // inputs
    double * label= (double*) mxGetPr(prhs[0]); 
    double * pressure = (double*) mxGetPr(prhs[1]); 

    const mwSize* dims = mxGetDimensions(prhs[0]);    
    int m = dims[0]; int n = dims[1]; int p = dims[2];
    double * maxiters = (double*) mxGetPr(prhs[2]);
    int maxIter = (int) maxiters[0];   
    printf("%d x %d x %d; maxIters: %d\n", m, n, p, maxIter);

    // outputs       
    plhs[0] = mxCreateNumericArray(3, dims, mxDOUBLE_CLASS, mxREAL);
    double * out = (double*) mxGetPr(plhs[0]);


    double * fx = (double*)malloc(sizeof(double)*m*n*p);
    double * fy = (double*)malloc(sizeof(double)*m*n*p);
    double * fz = (double*)malloc(sizeof(double)*m*n*p);

    for(int kk=0; kk<m*n*p; kk++)
	fx[kk]=0;
    for(int kk=0; kk<m*n*p; kk++)
	fy[kk]=0;
    for(int kk=0; kk<m*n*p; kk++)
	fz[kk]=0;


    int iter = 0;
    iter = flow3(label, pressure, fx, fy, fz, m, n, p, maxIter);    

    for(int kk=0; kk<m*n*p; kk++)
        out[kk]=pressure[kk];        

    plhs[1] = mxCreateNumericMatrix(1,1,mxDOUBLE_CLASS, mxREAL);
    double * niters = mxGetPr(plhs[1]);
    niters[0]=(double) iter;

    free(fx);
    free(fy);
    free(fz);

}
