#include "mex.h"
#include <math.h>
//#include <omp.h> // Comment out if not installed

/*
 * Compilation:
 * mex integralImage3.cpp CXXOPTIMFLAGS='-fopenmp -O3 -Wall' -largeArrayDims LDOPTIMFLAGS='-O3 -Wall -fopenmp'
 * 
*/

typedef unsigned short uint16;
typedef unsigned char uint8;

void mexFunction(
    int nlhs, mxArray *plhs[],
    int nrhs, const mxArray *prhs[])
{   

    if(nrhs!=3)
      {mexErrMsgTxt("Expecting three input arguments");}

    if(mxGetNumberOfDimensions(prhs[0])!=3)
      {mexErrMsgTxt("prhs0: Only for 3D input"); }

  mxClassID dataType;
  dataType = mxGetClassID(prhs[0]);
  if(dataType != mxUINT8_CLASS)
      {mexErrMsgTxt("prhs0: Only for uint8 input"); }

  dataType = mxGetClassID(prhs[1]);
  if(dataType != mxUINT16_CLASS)
      {mexErrMsgTxt("prhs1: Only for uint16 input"); }

  dataType = mxGetClassID(prhs[2]);
  if(dataType != mxUINT16_CLASS)
      {mexErrMsgTxt("prhs2: Only for uint16 input"); }

    const mwSize *dims;
    dims = mxGetDimensions(prhs[0]);
    int D1=dims[0]; 
    int D2=dims[1];
    int D3=dims[2];
  
    mwSize dimsOut[2];
    dimsOut[0]=(mwSize) D2;
    dimsOut[1]=(mwSize) D3;
       
    // Create output variables
    plhs[0] = mxCreateNumericArray(2, dimsOut, mxDOUBLE_CLASS, mxREAL);    
    double * out = mxGetPr(plhs[0]);

    uint8 * in = (uint8*) mxGetPr(prhs[0]);

  uint16* prhs1 = (uint16*) mxGetPr(prhs[1]);
  uint16* prhs2 = (uint16*) mxGetPr(prhs[2]);
 
uint16 firstSlice = (uint16) prhs1[0]; uint16 lastSlice = (uint16) prhs2[0]; 

  if(firstSlice < 1)
      {mexErrMsgTxt("First slice has to be > 0"); }
  if(firstSlice > D3)
      {mexErrMsgTxt("First slice has to be < size(V,3)"); }
  if(firstSlice > lastSlice)
      {mexErrMsgTxt("First slice has to be < last slice"); }

int spos;
int sum;
#pragma omp parallel for num_threads(omp_get_num_procs()) private(spos,sum)
for (int kk=0; kk<D2*D3; kk++) 
   {    
    spos = kk*D1+firstSlice; // start position
    sum = 0;
    for (int ll=0; ll<lastSlice-firstSlice; ll++)     
        sum+= in[spos++]; 
    out[kk]=sum;        
  }
}
