figure(3)
clf

% Display all demarcated fibres as tubes
% Using radius measurements if available
% radius measurements are created by contactAnalysis.m

s.plotTubes=1; % Set 1 for tubes
s.radius=5;   % Radius of tubes
s.npoints = 100; % Reduction of the number of points to be used, set low for higher speed
s.roundness = 9; % Set lower for speed

hold on
for kk=1:numel(fibres)    
    fprintf('Visualising fibre %d', kk);            
    
    if (numel(fibres{kk})>0)  % If not deleted
        if numel(fibres{kk}.xx>0) % If there are measurements
        
    txx=fibres{kk}.xx;
    tyy=fibres{kk}.yy;
    tzz=fibres{kk}.zz;
    
    if numel(txx)>0 % If non-zero length
    % Reduce the number of points
    txx = interp1(txx, linspace(1,numel(txx),s.npoints));
    tyy = interp1(tyy, linspace(1,numel(tyy),s.npoints));
    tzz = interp1(tzz, linspace(1,numel(tzz),s.npoints));
        
    if s.plotTubes                        
        % Set the color according to the mean z-position
        C = jet;
        r = interp1(linspace(0,1, size(C,1)), C(:,1)', mean(tzz)/size(V,2));
        g = interp1(linspace(0,1, size(C,1)), C(:,2)', mean(tzz)/size(V,2));
        b = interp1(linspace(0,1, size(C,1)), C(:,3)', mean(tzz)/size(V,2));
        if isnan(r) | isnan(g) | isnan(b)
            r=.5; g=r; b=r;
        end
        
        %tubeplot(txx,tyy,tzz,2*s.radius, [r,g,b]);        
        try
            rad = fibres{kk}.m.radius + 0.5*fibres{kk}.m.thickness;
                if rad==-1
                    disp(['No radius available for fibre ' kk]);
                    rad = s.radius;
                end
        catch e
            rad = s.radius;
        end           
            fprintf(', rad=%f\n', rad);
            if ~isreal(rad)
                 disp('Error: complex radius');
            else
                tubeplot(txx,tyy,tzz,rad, [r,g,b], s.roundness, [0,0,1]);
            end
        end    
    end 
        else
            disp(' -- no measurements available')
        end
    else
        disp(' -- deleted, skipping')        
    end
    
end

%axis([1, 1001, 1,1952, 1,101]);
axis vis3d equal off
rotate3d on

drawbb(size(V,3), size(V,2), size(V,1))

if s.plotTubes
   %colormap(jet)
   set(gcf, 'renderer', 'opengl')
   camlight 
   camlight headlight
   rotate3d
   lighting phong    
end

view(3)
set(gcf, 'Color', [0,0,0])