function [xx,yy,zz] = startTrack(vol, P, N)
P

% Start tracking a fibre in volume V(x,y,z)
% P defines the starting point
% N defines the normal direction

fig = figure(11);
set(fig, 'Pointer', 'Cross');
pl = subplot('position', [0,0.0,1,1]);
set(fig, 'WindowButtonMotionFcn', @showCoordinates);
set(fig, 'WindowButtonDownFcn', @windowClicked);


% Coordinates of the tracked fibre:
xx = [P(1)-N(1); P(1)];
yy = [P(2)-N(2); P(2)];
zz = [P(3)-N(3); P(3)];

% Settings:
step = 10; % Step length, in pixels
radius = 20; % Max radius of fibre
im = imagesc(ones(2*radius+1, 2*radius+1));
colormap gray
set(gca, 'Clim', [98,116]);

nx =0 ; ny = 0; nz=0; % Clicked points
pos = 2; % Position in the coordinate vectors
track = 1;
while track
    P = [xx(max(1, pos-5)), yy(max(1, pos-5)), zz(max(1, pos-5))]; % Previous point
    Q = [xx(pos), yy(pos), zz(pos)] % Last marked point
    
    D = Q-P; D=D/norm(D) % Direction
    
    Q2 = Q + D*step; % New point
        
    % U-V coordinate system, defines an ON basis together with D around Q
    U = [1,0,0]; U = U-(U*D')*D; U = U/norm(U)
    V = cross(U,D)
    
    [v, u]=meshgrid(-radius:radius, -radius:radius);
    X = u*U(1)+v*V(1)+Q2(1);
    Y = u*U(2)+v*V(2)+Q2(2);
    Z = u*U(3)+v*V(3)+Q2(3);
    %figure(12)
    %surface(X,Y,Z)
    figure(11)
    
    slice = trilinear8(vol, Y, X, Z);
        
    set(im, 'CData', slice);
    set(fig, 'Name', '');
    
    waitfor(fig, 'Name', 'ok');       
    % nx, ny and nz comes from the window interaction
    xx=[xx; nx];
    yy=[yy; ny];
    zz=[zz; nz];
    
        %disp('Q'), disp(Q)
    %disp('Q2'), disp(Q2)
    %disp('New'), disp([nx,ny,nz])
    
    pos = pos+1;
    %figure(17)
    %plot3(xx,yy,zz)
    figure(11)
    assignin('base', 'xx', zz)
    assignin('base', 'yy', yy)
    assignin('base', 'zz', xx)
end


    function showCoordinates(varargin)        
        pt = get(pl, 'CurrentPoint');
        v = pt(1,1)-radius; u = pt(1,2)-radius;
        nx = u*U(1)+v*V(1)+Q2(1);
        ny = u*U(2)+v*V(2)+Q2(2);
        nz = u*U(3)+v*V(3)+Q2(3);
        
        set(fig, 'name', sprintf('(%f, %f) (%f, %f, %f)', u, v, nx, ny, nz));          
    end

    function windowClicked(varargin)
        set(fig, 'Name', 'ok')
        pt = get(pl, 'CurrentPoint');
        v = pt(1,1)-radius; u = pt(1,2)-radius;
        
        nx = u*U(1)+v*V(1)+Q2(1);
        ny = u*U(2)+v*V(2)+Q2(2);
        nz = u*U(3)+v*V(3)+Q2(3);
        
    end

end