function [Y] = unitspeed(X, varargin)
% Re-interpolate the signal (x,y) so that the spacing is equal
% unitspeed(x,y,N) interpolate using N points

if ~exist('X','var')
    demo
    return
end

if size(X,1)==1
    X=X(:);
    disp('!')
end

if numel(varargin)==1
    nOutPoints = varargin{1};
else
    nOutPoints = size(X,1);
end

    npoints = 2*size(X,1); % Number of intermediate points
    T = zeros(npoints,size(X,2));
    Y = zeros(nOutPoints, size(X,2));
    
    for kk = 1:size(X,2)
        T(:,kk)=spline(1:size(X,1),X(:,kk), linspace(1,size(X,1),npoints));        
    end
    
    %figure, plot3(T(:,1), T(:,2), T(:,3), 'x')    
    
    % Get the speed
    l=zeros(size(T,1)-1,1);
    
    for kk=1:size(X,2)
        l = l + (T(2:end,kk)-T(1:end-1,kk)).^2;
    end
    l = [0; l];
    l = l.^(1/2);            
    cl = cumsum(l);

    % Invert the speed    
    icl = interp1(cl, 0:numel(cl)-1, linspace(0,max(cl),nOutPoints));
    %figure(2), plot(icl), hold on, plot(icll,'r');
    %figure, subplot(1,2,1), plot(cl), subplot(1,2,2), plot(icl)
    % Re-interpolate with unit speed
    icl = icl(:);
    whos
    for kk=1:size(X,2)
        Y(:,kk)=spline((0:size(T,1)-1), T(:,kk)', icl'/max(icl)*(size(T,1)-1));
    end
end

function demo
X = rand(4,3);

figure,
subplot(1,2,1)
for kk=1:size(X,2)
    XX(:,kk) = spline(1:size(X,1),X(:,kk), linspace(1,size(X,1),50));
end
plot3(XX(:,1),XX(:,2),XX(:,3), 'k');
hold on
plot3(XX(:,1),XX(:,2),XX(:,3), 'rx');
plot3(X(:,1),X(:,2),X(:,3),'o', 'MarkerSize', 15)
axis([0,1,0,1,0,1])
title('Input signal')
[Y]=unitspeed(XX, 50);

subplot(1,2,2)
plot3(Y(:,1),Y(:,2),Y(:,3), 'k')
hold on
plot3(Y(:,1),Y(:,2),Y(:,3), 'rx')
title('Unit speed output')
axis([0,1,0,1,0,1])

end

