inputNum = inputdlg(sprintf('Enter fibre number (1--%d)', numel(fibres)));
vs.fibreNo=str2num(inputNum{1});

vs.fibreNo = 1;
vs.maxRadius = 20; % radius around marked fibres
vs.padding = vs.maxRadius + 3;
vs.visible = 'on'; % on or off
vs.constColor = 1; % one or two colors (slower)
vs.colorMode = 2; % if vs.constColor == 0
vs.saveToDisk = 0;

disp('Single fibre visualisation settings:')
disp(vs);

% 1 - red-blue
if vs.colorMode ==1;
    vs.backgroundColor = [.9,.9, .9];
    vs.frameColor = [0,0,0];
    vs.colormap = [[.9,.3,.3]; [.3,.3,.9]];
end

% 2 - blue-blue
if vs.colorMode ==2;
    vs.backgroundColor = [.9,.9, .9];
    vs.frameColor = [0,0,0];
    vs.colormap = [[.1,.1,.25]; [.3,.3,.9]];
end
    
% 3 - whiteish - dark
if vs.colorMode == 3
    vs.backgroundColor = [.1,.1, .1];
    vs.frameColor = [1,1,1];
    vs.colormap = [[.90,.95,.7]/3
                  [.90,.95,.7]];
end

try
    close(vfig)
end
% permute to match the volume dimensions
x = fibres{vs.fibreNo}.zz;
y = fibres{vs.fibreNo}.yy;
z = fibres{vs.fibreNo}.xx;

%x = x(1:100); y = y(1:100); z = z(1:100);

%% Bounding box, to same some memory and gain some speed
pad = vs.padding;
bb = [max(1, round(min(x))-pad), min(max(round(x))+pad, size(V,1)), ...
      max(1, round(min(y))-pad), min(max(round(y))+pad, size(V,2)), ...
      max(1, round(min(z))-pad), min(max(round(z))+pad, size(V,3))];

% Convert x,y and z to the bounding box
xb = x-bb(1); yb = y-bb(3); zb = z-bb(5);
% extract the ROI
ROI = V(bb(1):bb(2), bb(3):bb(4), bb(5):bb(6));
ROImask = zeros(size(ROI));

for(kk=1:numel(x))
    try
        ROImask(round(xb(kk)),round(yb(kk)),round(zb(kk)))=255;
    end
end

ROImask = bwdist(ROImask, 'quasi-euclidean');
Dist = ROImask;
ROImask = double(erfclip(ROImask, vs.maxRadius, 2));
ROImask = .5-(ROImask-vs.maxRadius)/2;

ROI = uint8(double(ROI).*ROImask);
%volumeSlide(ROI)

%% Export and open with paraview or something similar
[f,v]=isosurface(ROI, 128);
[f2,v2]=isosurface(ROImask, .5);
d = interpn(Dist, v(:,2), v(:,1), v(:,3));
d = d/(vs.maxRadius-1);
d = 1-d.^19;
d(d<0)=0;

vfig = figure('visible', vs.visible);
set(vfig, 'renderer', 'opengl')
%v(:,1)=v(:,1)+bb(3)-1;
%v(:,2)=v(:,2)+bb(1)-1;
%v(:,3)=v(:,3)+bb(5)-1;

% Constant coloring is slightly faster to rotate...
if vs.constColor == 0
    p = patch('faces', f, ...
    'vertices', v, ... %    'FaceColor', [.2,.2,.7], ... %
    'FaceColor', 'interp', 'FaceVertexCData', d, ...
    'EdgeAlpha', 0, ...
    'AmbientStrength', .8, ...
    'DiffuseStrength', .8);
else
    p = patch('faces', f, ...
    'vertices', v, ... %    
    'FaceColor', [.3,.3,.9], ... %    
    'EdgeAlpha', 0, ...
    'AmbientStrength', .8, ...
    'DiffuseStrength', .8);
end

hold on
colormap(vs.colormap); % blue-blue

set(gca, 'clim', [0,1])

isonormals(ROI, p);
lighting gouraud
material dull
set(vfig, 'Color', vs.backgroundColor)
drawbg(size(ROI,2), size(ROI,1), size(ROI,3), vs.frameColor);
axis off
axis equal
view(3)

hl = camlight('left');
r3d = rotate3d();
set(r3d, 'actionpostcallback', 'camlight(hl, ''right'')');
set(r3d, 'enable', 'on');
zoom(1);
if vs.saveToDisk
    print('-dpng', sprintf('sfibre%03d.png', fibreNo'));
    saveas(fig, sprintf('sfibre%03d.fig', fibreNo'), 'fig');
end